@@ -2,6 +2,7 @@
   Copyright (c) 2004 Jan Schaefer <j_schaef@informatik.uni-kl.de>
   Copyright (c) 2011 Rodrigo Belem <rclbelem@gmail.com>
   Copyright (c) 2015 Harald Sitter <sitter@kde.org>
+  Copyright (c) 2019 Nate Graham <nate@kde.org>
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -32,6 +33,7 @@
 #include <KPluginLoader>
 #include <KSambaShare>
 #include <KSambaShareData>
+#include <ktoolinvocation.h>
 
 #include "sambausershareplugin.h"
 #include "model.h"
@@ -89,13 +91,16 @@ SambaUserSharePlugin::SambaUserSharePlugin(QObject *parent, const QList<QVariant
     vLayout->setAlignment(Qt::AlignJustify);
     vLayout->setMargin(0);
 
-    vLayout->addWidget(new QLabel(i18n("Samba is not installed on your system."), m_installSambaWidgets));
+    m_sambaStatusMessage = new QLabel(i18n("Samba must be installed before folders can be shared."));
+    m_sambaStatusMessage->setAlignment(Qt::AlignCenter);
+    vLayout->addWidget(m_sambaStatusMessage);
 
 #ifdef SAMBA_INSTALL
-    m_installSambaButton = new QPushButton(i18n("Install Samba..."), m_installSambaWidgets);
+    m_installSambaButton = new QPushButton(i18n("Install Samba"), m_installSambaWidgets);
     m_installSambaButton->setDefault(false);
     vLayout->addWidget(m_installSambaButton);
-    connect(m_installSambaButton, SIGNAL(clicked()), SLOT(installSamba()));
+    connect(m_installSambaButton, &QPushButton::clicked,
+            this, &SambaUserSharePlugin::installSamba);
     m_installProgress = new QProgressBar();
     vLayout->addWidget(m_installProgress);
     m_installProgress->hide();
@@ -117,16 +122,21 @@ SambaUserSharePlugin::SambaUserSharePlugin(QObject *parent, const QList<QVariant
     setupViews();
     load();
 
-    connect(propertiesUi.sambaChk, SIGNAL(toggled(bool)), this, SLOT(toggleShareStatus(bool)));
-    connect(propertiesUi.sambaChk, SIGNAL(toggled(bool)), this, SIGNAL(changed()));
-    connect(propertiesUi.sambaNameEdit, SIGNAL(textChanged(QString)), this, SIGNAL(changed()));
-    connect(propertiesUi.sambaNameEdit, SIGNAL(textChanged(QString)), this, SLOT(checkShareName(QString)));
+    connect(propertiesUi.sambaChk, &QCheckBox::toggled,
+            this, &SambaUserSharePlugin::toggleShareStatus);
+    connect(propertiesUi.sambaNameEdit, &QLineEdit::textChanged,
+            this, &SambaUserSharePlugin::checkShareName);
     connect(propertiesUi.sambaAllowGuestChk, &QCheckBox::toggled,
             this, [=] (bool checked) {
                 propertiesUi.tableView->setEnabled(checked && propertiesUi.sambaChk->isChecked());
                 setDirty();
             });
-    connect(model, SIGNAL(dataChanged(QModelIndex,QModelIndex)), this, SIGNAL(changed()));
+    connect(model, &UserPermissionModel::dataChanged,
+            this, [=] { setDirty(); });
+    connect(propertiesUi.sambaStatusMonitorButton, &QPushButton::clicked,
+            this, [] {
+                KToolInvocation::kdeinitExec(QStringLiteral("kcmshell5"), QStringList() << QStringLiteral("smbstatus"));
+            });
 
     for (int i = 0; i < model->rowCount(); ++i) {
         propertiesUi.tableView->openPersistentEditor(model->index(i, 1, QModelIndex()));
@@ -148,29 +158,31 @@ SambaUserSharePlugin::~SambaUserSharePlugin()
 void SambaUserSharePlugin::installSamba()
 {
     QString package = QStringLiteral(SAMBA_PACKAGE_NAME);
-    PackageKit::Transaction *transaction = PackageKit::Daemon::resolve(package,
-                                                   PackageKit::Transaction::FilterArch);
-    connect(transaction,
-            SIGNAL(package(PackageKit::Transaction::Info,QString,QString)),
-            SLOT(packageInstall(PackageKit::Transaction::Info,QString,QString)));
+    QStringList distroSambaPackages = package.split(QLatin1Char(','));
+
+    PackageKit::Transaction *transaction = PackageKit::Daemon::resolve(distroSambaPackages, PackageKit::Transaction::FilterArch);
+
+    QSharedPointer<QStringList> pkgids(new QStringList);
+
+    connect(transaction, &PackageKit::Transaction::package,
+            this, [pkgids] (PackageKit::Transaction::Info /*info*/, const QString& packageId, const QString& /*summary*/) { pkgids->append(packageId); });
+
+    connect(transaction, &PackageKit::Transaction::finished,
+            this, [this, pkgids] (PackageKit::Transaction::Exit exit) {
+                if (exit != PackageKit::Transaction::ExitSuccess) { return; }
+                auto installTransaction = PackageKit::Daemon::installPackages(*pkgids);
+                connect(installTransaction, &PackageKit::Transaction::finished,
+                        this, &SambaUserSharePlugin::packageFinished);
+            }
+    );
+
+    m_sambaStatusMessage->setText(i18n("Installing Samba..."));
     m_installProgress->setMaximum(0);
     m_installProgress->setMinimum(0);
     m_installProgress->show();
     m_installSambaButton->hide();
 }
 
-void SambaUserSharePlugin::packageInstall(PackageKit::Transaction::Info info,
-                                          const QString &packageId,
-                                          const QString &summary)
-{
-    Q_UNUSED(info);
-    Q_UNUSED(summary);
-    PackageKit::Transaction *installTransaction = PackageKit::Daemon::installPackage(packageId);
-    connect(installTransaction,
-            SIGNAL(finished(PackageKit::Transaction::Exit, uint)),
-            SLOT(packageFinished(PackageKit::Transaction::Exit, uint)));
-}
-
 void SambaUserSharePlugin::packageFinished(PackageKit::Transaction::Exit status, uint runtime)
 {
     Q_UNUSED(runtime);
@@ -253,6 +265,7 @@ void SambaUserSharePlugin::toggleShareStatus(bool checked)
     } else {
         propertiesUi.sambaNameEdit->setText(QString());
     }
+    setDirty();
 }
 
 void SambaUserSharePlugin::checkShareName(const QString &name)
@@ -283,6 +296,7 @@ void SambaUserSharePlugin::checkShareName(const QString &name)
 
     if (!properties->button(QDialogButtonBox::Ok)->isEnabled()) {
         properties->button(QDialogButtonBox::Ok)->setEnabled(true);
+        setDirty();
     }
 }
 
